// RelativeLayers 0.9.7
// 
// $Id: rl_limits.js 1.12 04/02/02 21:22:32+01:00 gbevin@willow.uwyn.office $
// 
// Library for creating dynamic designs that adapt themselves to the user's
// browser environment.
// 
// Copyright (C) 1998-2004, Geert Bevin
// Distributed under the terms of the GNU Lesser General Public, v2.1 or later
// 
// gbevin[remove] at uwyn dot com
// http://www.uwyn.com/projects/relativelayers

// define the positioning limit edges
var TOPEDGE = 0;
var BOTTOMEDGE = 1;
var LEFTEDGE = 2;
var RIGHTEDGE = 3;

// define the dimension limits
var WIDTH = 0;
var HEIGHT = 1;

// define the valid positioning identifiers
var ABOVE = 0;
var BELOW = 1;
var LEFTSIDE = 2;
var RIGHTSIDE = 3;

// define the valid dimension identifiers
var MINIMUM = 0;
var MAXIMUM = 1;

extendMethod("calculateVisibleHorizOffset", RelativeLayer.prototype.calculateVisibleHorizOffset, calculateVisibleHorizOffset_extensionlimit);
extendMethod("calculateVisibleVertOffset", RelativeLayer.prototype.calculateVisibleVertOffset, calculateVisibleVertOffset_extensionlimit);
extendMethod("calculateVisibleWidth", RelativeLayer.prototype.calculateVisibleWidth, calculateVisibleWidth_extensionlimit);
extendMethod("calculateVisibleHeight", RelativeLayer.prototype.calculateVisibleHeight, calculateVisibleHeight_extensionlimit);

RelativeLayer.prototype.addLimit = function(par_limitedEdge, par_validZone, par_limitingLayer, par_limitingLayerBorder, par_limitingLayerBorderOffset)
{
	if (this.css)
	{
		if (!this.mbr_limits)
		{
			this.mbr_limits = new Array();
		}

		this.mbr_limits.push(new Array(par_limitedEdge, par_validZone, par_limitingLayer, par_limitingLayerBorder, par_limitingLayerBorderOffset));
		if (par_limitingLayer != "")
		{
			this.addDependency(par_limitingLayer);
		}
		this.setPositioned(false);
	}
	else
	{
		this.deferCommand("this.addLimit("+par_limitedEdge+","+par_validZone+", \""+par_limitingLayer+"\", "+par_limitingLayerBorder+", "+par_limitingLayerBorderOffset+")");
	}
};

RelativeLayer.prototype.addDimensionLimit = function(par_limitedDimension, par_validExtreme, par_limitingLayer1, par_limitingLayer1Border, par_limitingLayer1BorderOffset, par_limitingLayer2, par_limitingLayer2Border, par_limitingLayer2BorderOffset)
{
	if (this.css)
	{
		if (!this.mbr_dimensionlimits)
		{
			this.mbr_dimensionlimits = new Array();
		}

		this.mbr_dimensionlimits.push(new Array(par_limitedDimension, par_validExtreme, par_limitingLayer1, par_limitingLayer1Border, par_limitingLayer1BorderOffset, par_limitingLayer2, par_limitingLayer2Border, par_limitingLayer2BorderOffset));
		if (par_limitingLayer1 != "")
		{
			this.addDependency(par_limitingLayer1);
		}
		if (par_limitingLayer2 != "")
		{
			this.addDependency(par_limitingLayer2);
		}
		this.setPositioned(false);
	}
	else
	{
		this.deferCommand("this.addDimensionLimit("+par_limitedDimension+","+par_validExtreme+", \""+par_limitingLayer1+"\", "+par_limitingLayer1Border+", "+par_limitingLayer1BorderOffset+", \""+par_limitingLayer2+"\", "+par_limitingLayer2Border+", "+par_limitingLayer2BorderOffset+")");
	}
};

RelativeLayer.prototype.clearLimits = function()
{
	if (this.css)
	{
		if (this.mbr_limits)
		{
			var var_index;

			for (var_index = 0; var_index < this.mbr_limits.length; var_index++)
			{
				if (this.mbr_limits[var_index][2] != "")
				{
					this.removeDependency(this.mbr_limits[var_index][2]);
				}
			}
			this.mbr_limits.length = 0;
			this.setPositioned(false);
		}
		if (this.mbr_dimensionlimits)
		{
			var var_index;

			for (var_index = 0; var_index < this.mbr_dimensionlimits.length; var_index++)
			{
				if (this.mbr_dimensionlimits[var_index][2] != "")
				{
					this.removeDependency(this.mbr_dimensionlimits[var_index][2]);
				}
				if (this.mbr_dimensionlimits[var_index][5] != "")
				{
					this.removeDependency(this.mbr_dimensionlimits[var_index][5]);
				}
			}
			this.mbr_dimensionlimits.length = 0;
			this.setPositioned(false);
		}
	}
	else
	{
		this.deferCommand("this.clearLimits()");
	}
};

function calculateVisibleHorizOffset_extensionlimit()
{
	if (this.mbr_limits)
	{
		// apply the defined limits
		for (var var_limitrule = 0; var_limitrule < this.mbr_limits.length; var_limitrule++)
		{
			if (this.mbr_limits[var_limitrule][0] == LEFTEDGE ||
			   this.mbr_limits[var_limitrule][0] == RIGHTEDGE)
			{
				var var_limitlayer_id = this.mbr_limits[var_limitrule][2];
				var var_limitlayer_visiblehorizoffset;
				var var_limitlayer_visiblewidth;
	
				if (var_limitlayer_id == "")
				{
					// limit is defined by the window visible area
					var_limitlayer_visiblehorizoffset = 0;
					var_limitlayer_visiblewidth = getWindowWidth();
				}
				else
				{
					// limit is defined by another layer
					var var_limitlayer = getRelativeLayer(var_limitlayer_id);
					var_limitlayer_visiblehorizoffset = var_limitlayer.calculateVisibleHorizOffset();
					var_limitlayer_visiblewidth = var_limitlayer.calculateVisibleWidth();
				}
	
				if (this.mbr_limits[var_limitrule][0] == LEFTEDGE)
				{
					if (this.mbr_limits[var_limitrule][1] == LEFTSIDE && this.mbr_limits[var_limitrule][3] == LEFT)
					{
						var var_limitlayer_leftedge = var_limitlayer_visiblehorizoffset+parseInt(this.mbr_limits[var_limitrule][4]);
	
						if (var_visiblehorizoffset > var_limitlayer_leftedge)
						{
							var_visiblehorizoffset = var_limitlayer_leftedge;
						}
					}
					else if (this.mbr_limits[var_limitrule][1] == RIGHTSIDE && this.mbr_limits[var_limitrule][3] == LEFT)
					{
						var var_limitlayer_leftedge = var_limitlayer_visiblehorizoffset+parseInt(this.mbr_limits[var_limitrule][4]);
	
						if (var_visiblehorizoffset < var_limitlayer_leftedge)
						{
							var_visiblehorizoffset = var_limitlayer_leftedge;
						}
					}
					else if (this.mbr_limits[var_limitrule][1] == LEFTSIDE && this.mbr_limits[var_limitrule][3] == RIGHT)
					{
						var var_limitlayer_rightedge = var_limitlayer_visiblehorizoffset+var_limitlayer_visiblewidth+parseInt(this.mbr_limits[var_limitrule][4]);
	
						if (var_visiblehorizoffset > var_limitlayer_rightedge)
						{
							var_visiblehorizoffset = var_limitlayer_rightedge;
						}
					}
					else if (this.mbr_limits[var_limitrule][1] == RIGHTSIDE && this.mbr_limits[var_limitrule][3] == RIGHT)
					{
						var var_limitlayer_rightedge = var_limitlayer_visiblehorizoffset+var_limitlayer_visiblewidth+parseInt(this.mbr_limits[var_limitrule][4]);
	
						if (var_visiblehorizoffset < var_limitlayer_rightedge)
						{
							var_visiblehorizoffset = var_limitlayer_rightedge;
						}
					}
				}
				else if (this.mbr_limits[var_limitrule][0] == RIGHTEDGE)
				{
					var var_visible_rightedge = var_visiblehorizoffset + var_visiblewidth; 
	
					if (this.mbr_limits[var_limitrule][1] == LEFTSIDE && this.mbr_limits[var_limitrule][3] == LEFT)
					{
						var var_limitlayer_leftedge = var_limitlayer_visiblehorizoffset+parseInt(this.mbr_limits[var_limitrule][4]);
	
						if (var_visible_rightedge > var_limitlayer_leftedge)
						{
							var_visiblehorizoffset = var_limitlayer_leftedge - var_visiblewidth;
						}
					}
					else if (this.mbr_limits[var_limitrule][1] == RIGHTSIDE && this.mbr_limits[var_limitrule][3] == LEFT)
					{
						var var_limitlayer_leftedge = var_limitlayer_visiblehorizoffset+parseInt(this.mbr_limits[var_limitrule][4]);
	
						if (var_visible_rightedge < var_limitlayer_leftedge)
						{
							var_visiblehorizoffset = var_limitlayer_leftedge - var_visiblewidth;
						}
					}
					else if (this.mbr_limits[var_limitrule][1] == LEFTSIDE && this.mbr_limits[var_limitrule][3] == RIGHT)
					{
						var var_limitlayer_rightedge = var_limitlayer_visiblehorizoffset+var_limitlayer_visiblewidth+parseInt(this.mbr_limits[var_limitrule][4]);
	
						if (var_visible_rightedge > var_limitlayer_rightedge)
						{
							var_visiblehorizoffset = var_limitlayer_rightedge - var_visiblewidth;
						}
					}
					else if (this.mbr_limits[var_limitrule][1] == RIGHTSIDE && this.mbr_limits[var_limitrule][3] == RIGHT)
					{
						var var_limitlayer_rightedge = var_limitlayer_visiblehorizoffset+var_limitlayer_visiblewidth+parseInt(this.mbr_limits[var_limitrule][4]);
	
						if (var_visible_rightedge < var_limitlayer_rightedge)
						{
							var_visiblehorizoffset = var_limitlayer_rightedge - var_visiblewidth;
						}
					}
				}
			}
		}
	}
};

function calculateVisibleVertOffset_extensionlimit()
{
	if (this.mbr_limits)
	{
		// apply the defined limits
		for (var var_limitrule = 0; var_limitrule < this.mbr_limits.length; var_limitrule++)
		{
			if (this.mbr_limits[var_limitrule][0] == TOPEDGE ||
			   this.mbr_limits[var_limitrule][0] == BOTTOMEDGE)
			{
				var var_limitlayer_id = this.mbr_limits[var_limitrule][2];
				var var_limitlayer_visiblevertoffset;
				var var_limitlayer_visibleheight;
				
				if (var_limitlayer_id == "")
				{
					// limit is defined by the window visible area
					var_limitlayer_visiblevertoffset = 0;
					var_limitlayer_visibleheight = getWindowHeight();
				}
				else
				{
					// limit is defined by another layer
					var var_limitlayer = getRelativeLayer(var_limitlayer_id);
					var_limitlayer_visiblevertoffset = var_limitlayer.calculateVisibleVertOffset();
					var_limitlayer_visibleheight = var_limitlayer.calculateVisibleHeight();
				}
				
				if (this.mbr_limits[var_limitrule][0] == TOPEDGE)
				{
					if (this.mbr_limits[var_limitrule][1] == ABOVE && this.mbr_limits[var_limitrule][3] == TOP)
					{
						var var_limitlayer_topedge = var_limitlayer_visiblevertoffset+parseInt(this.mbr_limits[var_limitrule][4]);
						
						if (var_visiblevertoffset > var_limitlayer_topedge)
						{
							var_visiblevertoffset = var_limitlayer_topedge;
						}
					}
					else if (this.mbr_limits[var_limitrule][1] == BELOW && this.mbr_limits[var_limitrule][3] == TOP)
					{
						var var_limitlayer_topedge = var_limitlayer_visiblevertoffset+parseInt(this.mbr_limits[var_limitrule][4]);
						
						if (var_visiblevertoffset < var_limitlayer_topedge)
						{
							var_visiblevertoffset = var_limitlayer_topedge;
						}
					}
					else if (this.mbr_limits[var_limitrule][1] == ABOVE && this.mbr_limits[var_limitrule][3] == BOTTOM)
					{
						var var_limitlayer_bottomedge = var_limitlayer_visiblevertoffset+var_limitlayer_visibleheight+parseInt(this.mbr_limits[var_limitrule][4]);
						
						if (var_visiblevertoffset > var_limitlayer_bottomedge)
						{
							var_visiblevertoffset = var_limitlayer_bottomedge;
						}
					}
					else if (this.mbr_limits[var_limitrule][1] == BELOW && this.mbr_limits[var_limitrule][3] == BOTTOM)
					{
						var var_limitlayer_bottomedge = var_limitlayer_visiblevertoffset+var_limitlayer_visibleheight+parseInt(this.mbr_limits[var_limitrule][4]);
						
						if (var_visiblevertoffset < var_limitlayer_bottomedge)
						{
							var_visiblevertoffset = var_limitlayer_bottomedge;
						}
					}
				}
				else if (this.mbr_limits[var_limitrule][0] == BOTTOMEDGE)
				{
					var var_visible_bottomedge = var_visiblevertoffset + var_visibleheight; 
					
					if (this.mbr_limits[var_limitrule][1] == ABOVE && this.mbr_limits[var_limitrule][3] == TOP)
					{
						var var_limitlayer_topedge = var_limitlayer_visiblevertoffset+parseInt(this.mbr_limits[var_limitrule][4]);
						
						if (var_visible_bottomedge > var_limitlayer_topedge)
						{
							var_visiblevertoffset = var_limitlayer_topedge - var_visibleheight;
						}
					}
					else if (this.mbr_limits[var_limitrule][1] == BELOW && this.mbr_limits[var_limitrule][3] == TOP)
					{
						var var_limitlayer_topedge = var_limitlayer_visiblevertoffset+parseInt(this.mbr_limits[var_limitrule][4]);
						
						if (var_visible_bottomedge < var_limitlayer_topedge)
						{
							var_visiblevertoffset = var_limitlayer_topedge - var_visibleheight;
						}
					}
					else if (this.mbr_limits[var_limitrule][1] == ABOVE && this.mbr_limits[var_limitrule][3] == BOTTOM)
					{
						var var_limitlayer_bottomedge = var_limitlayer_visiblevertoffset+var_limitlayer_visibleheight+parseInt(this.mbr_limits[var_limitrule][4]);
						
						if (var_visible_bottomedge > var_limitlayer_bottomedge)
						{
							var_visiblevertoffset = var_limitlayer_bottomedge - var_visibleheight;
						}
					}
					else if (this.mbr_limits[var_limitrule][1] == BELOW && this.mbr_limits[var_limitrule][3] == BOTTOM)
					{
						var var_limitlayer_bottomedge = var_limitlayer_visiblevertoffset+var_limitlayer_visibleheight+parseInt(this.mbr_limits[var_limitrule][4]);
						
						if (var_visible_bottomedge < var_limitlayer_bottomedge)
						{
							var_visiblevertoffset = var_limitlayer_bottomedge - var_visibleheight;
						}
					}
				}
			}
		}
	}
};

RelativeLayer.prototype.calculateDimensionLimit = function(par_limitArray)
{
	var var_limitlayer1_id = par_limitArray[2];
	var var_limitlayer1_visiblehorizoffset;
	var var_limitlayer1_visiblewidth;
	var var_limitlayer1_visiblevertoffset;
	var var_limitlayer1_visibleheight;
	
    var var_limitlayer2_id = par_limitArray[5];
	var var_limitlayer2_visiblehorizoffset;
	var var_limitlayer2_visiblewidth;
	var var_limitlayer2_visiblevertoffset;
	var var_limitlayer2_visibleheight;

	if (var_limitlayer1_id == "")
	{
		// prevent calculation if it isn't needed
		if (par_limitArray[3] == LEFT ||
			par_limitArray[3] == RIGHT)
		{
		// limit is defined by the window visible area
			var_limitlayer1_visiblehorizoffset = 0;
			var_limitlayer1_visiblewidth = getWindowWidth();
		}
		// prevent calculation if it isn't needed
		else if (par_limitArray[3] == TOP ||
			par_limitArray[3] == BOTTOM)
		{
			var_limitlayer1_visiblevertoffset = 0;
			var_limitlayer1_visibleheight = getWindowHeight();
		}
	}
	else
	{
		// limit is defined by another layer
		var var_limitlayer1 = getRelativeLayer(var_limitlayer1_id);
		
		// prevent calculation if it isn't needed
		if (par_limitArray[3] == LEFT ||
			par_limitArray[3] == RIGHT)
		{
			var_limitlayer1_visiblehorizoffset = var_limitlayer1.calculateVisibleHorizOffset();
			var_limitlayer1_visiblewidth = var_limitlayer1.calculateVisibleWidth();
		}
		// prevent calculation if it isn't needed
		else if (par_limitArray[3] == TOP ||
			par_limitArray[3] == BOTTOM)
		{
			var_limitlayer1_visiblevertoffset = var_limitlayer1.calculateVisibleVertOffset();
			var_limitlayer1_visibleheight = var_limitlayer1.calculateVisibleHeight();
		}
	}
	
	if (var_limitlayer2_id == "")
	{
		// prevent calculation if it isn't needed
		if (par_limitArray[6] == LEFT ||
			par_limitArray[6] == RIGHT)
		{
			// limit is defined by the window visible area
			var_limitlayer2_visiblehorizoffset = 0;
			var_limitlayer2_visiblewidth = getWindowWidth();
		}
		// prevent calculation if it isn't needed
		else if (par_limitArray[6] == TOP ||
			par_limitArray[6] == BOTTOM)
		{
			var_limitlayer2_visiblevertoffset = 0;
			var_limitlayer2_visibleheight = getWindowHeight();
		}
	}
	else
	{
		// limit is defined by another layer
		var var_limitlayer2 = getRelativeLayer(var_limitlayer2_id);

		// prevent calculation if it isn't needed
		if (par_limitArray[6] == LEFT ||
			par_limitArray[6] == RIGHT)
		{
			var_limitlayer2_visiblehorizoffset = var_limitlayer2.calculateVisibleHorizOffset();
			var_limitlayer2_visiblewidth = var_limitlayer2.calculateVisibleWidth();
		}
		// prevent calculation if it isn't needed
		else if (par_limitArray[6] == TOP ||
			par_limitArray[6] == BOTTOM)
		{
			var_limitlayer2_visiblevertoffset = var_limitlayer2.calculateVisibleVertOffset();
			var_limitlayer2_visibleheight = var_limitlayer2.calculateVisibleHeight();
		}
	}
	
	var var_limitedge1;
	var var_limitedge2;
	var var_dimensionlimit;

	if (par_limitArray[3] == LEFT)
    {
		var_limitedge1 = var_limitlayer1_visiblehorizoffset+parseInt(par_limitArray[4]);
    }
	if (par_limitArray[3] == RIGHT)
    {
		var_limitedge1 = var_limitlayer1_visiblehorizoffset+var_limitlayer1_visiblewidth+parseInt(par_limitArray[4]);
    }
	if (par_limitArray[6] == LEFT)
    {
		var_limitedge2 = var_limitlayer2_visiblehorizoffset+parseInt(par_limitArray[7]);
    }
	if (par_limitArray[6] == RIGHT)
    {
		var_limitedge2 = var_limitlayer2_visiblehorizoffset+var_limitlayer2_visiblewidth+parseInt(par_limitArray[7]);
    }
	if (par_limitArray[3] == TOP)
    {
		var_limitedge1 = var_limitlayer1_visiblevertoffset+parseInt(par_limitArray[4]);
    }
	if (par_limitArray[3] == BOTTOM)
    {
		var_limitedge1 = var_limitlayer1_visiblevertoffset+var_limitlayer1_visibleheight+parseInt(par_limitArray[4]);
    }
	if (par_limitArray[6] == TOP)
    {
		var_limitedge2 = var_limitlayer2_visiblevertoffset+parseInt(par_limitArray[7]);
    }
	if (par_limitArray[6] == BOTTOM)
    {
		var_limitedge2 = var_limitlayer2_visiblevertoffset+var_limitlayer2_visibleheight+parseInt(par_limitArray[7]);
    }
	var_dimensionlimit = Math.abs(var_limitedge2-var_limitedge1);
	
	return var_dimensionlimit;
};

function calculateVisibleWidth_extensionlimit()
{
	if (this.mbr_dimensionlimits)
	{
		// apply the defined limits
		for (var var_limitrule = 0; var_limitrule < this.mbr_dimensionlimits.length; var_limitrule++)
		{
			if (this.mbr_dimensionlimits[var_limitrule][0] == WIDTH)
			{
				var var_limitwidth = this.calculateDimensionLimit(this.mbr_dimensionlimits[var_limitrule]);

				if (this.mbr_dimensionlimits[var_limitrule][1] == MINIMUM &&
					var_visibleWidth < var_limitwidth)
				{
					var_visibleWidth = var_limitwidth;
				}
                
				if (this.mbr_dimensionlimits[var_limitrule][1] == MAXIMUM &&
					var_visibleWidth > var_limitwidth)
				{
					var_visibleWidth = var_limitwidth;
				}
            }
        }
    }
};

function calculateVisibleHeight_extensionlimit()
{
	if (this.mbr_dimensionlimits)
	{
		// apply the defined limits
		for (var var_limitrule = 0; var_limitrule < this.mbr_dimensionlimits.length; var_limitrule++)
		{
			if (this.mbr_dimensionlimits[var_limitrule][0] == HEIGHT)
			{
				var var_limitheight = this.calculateDimensionLimit(this.mbr_dimensionlimits[var_limitrule]);

				if (this.mbr_dimensionlimits[var_limitrule][1] == MINIMUM &&
					var_visibleHeight < var_limitheight)
				{
					var_visibleHeight = var_limitheight;
				}
                
				if (this.mbr_dimensionlimits[var_limitrule][1] == MAXIMUM &&
					var_visibleHeight > var_limitheight)
				{
					var_visibleHeight = var_limitheight;
				}
            }
        }
    }
};
