// RelativeLayers 0.9.7
// 
// $Id: rl_source.js 1.8 04/02/02 21:22:32+01:00 gbevin@willow.uwyn.office $
// 
// Library for creating dynamic designs that adapt themselves to the user's
// browser environment.
// 
// Copyright (C) 1998-2004, Geert Bevin
// Distributed under the terms of the GNU Lesser General Public, v2.1 or later
// 
// gbevin[remove] at uwyn dot com
// http://www.uwyn.com/projects/relativelayers

// variable to indicate wether the browser is busy updating a specific document
// since it's better to handle only one at a time (netscape doesn't support
// parallel updates anyhow)
var glb_updating_layer = null;

addLoadHook(initSources, 30);
addResizeHook(resizeUpdateSources, 20);

function initSources()
{
	var var_layers = glb_relativelayer_objects.values();
	var var_index;
	
	for (var_index = 0; var_index < var_layers.length; var_index++)
	{
		if (var_layers[var_index].isUpdateContentsCompleted &&
		   !var_layers[var_index].isUpdateContentsCompleted())
		{
			setTimeout("initSources()", 200);
			return false;
		}
	}

	nextHook();
	return true;
};

function resizeUpdateSources()
{
	var var_layers = glb_relativelayer_objects.values();
	var var_index;

	for (var_index = 0; var_index < var_layers.length; var_index++)
	{
		if (ns)
		{
			var_layers[var_index].mbr_updateCompleted = false;
			var_layers[var_index].updateContents();
		}
	}

	return true;
};

function finishContentsUpdate()
{
	if (glb_updating_layer &&
	   (ns || mz || kq))
	{
		glb_updating_layer.finishContentsUpdate();
	}
};

RelativeLayer.prototype.setSource = function(par_source)
{
	if (op)
	{
		unsupportedBrowser();
	}
	else
	{
		if (this.css)
		{
			this.mbr_source = par_source;
			this.updateContents();
		}
		else
		{
			this.deferCommand("this.setSource(\""+par_source+"\")");
		}
	}
};

RelativeLayer.prototype.isUpdateContentsCompleted = function()
{
	var var_completed = true;

	if (this.mbr_source &&
		this.mbr_source != "")
	{
		var_completed = this.mbr_updateCompleted;
	}

	return var_completed;
};

RelativeLayer.prototype.updateContents = function()
{
	if (this.mbr_source &&
		this.mbr_source != "")
	{
		// only update one content at a time
		if (glb_updating_layer)
		{
			setTimeout("getRelativeLayer('"+this.getId()+"').updateContents();", 200);
		}
		else
		{
			// notify event listerens
			this.fireEvent("beforeload");

			glb_updating_layer = this;
			this.mbr_updateCompleted = false;
			
			if (op)
			{
				// NOT IMPLEMENTED
			}
			else if (mz)
			{
				// check if the hidden iframe into which the contents will be downloaded exists, if not, create it
				this.checkIFrame();

				// Update contents of the element
				// check wether the URL has changed or not, since the contents don't have to be reloaded to update the wrapping of text
				if (document.getElementById(this.getId()+"Target"))
				{
					document.getElementById(this.getId()+"Target").src = this.mbr_source;
				}
			}
			else if (kq)
			{
				// check if the hidden iframe into which the contents will be downloaded exists, if not, create it
				this.replaceIFrame(this.mbr_source);
			}
			else if (ns)
			{
				// update the source of the layer and set the textwrap for Netscape
				this.css.load(this.mbr_source, this.calculateAvailableWidth());
				// don't work with an onload event here since the contents will be transferred, but the document will not yet be formatted and dimensioned
			}
			else if (ie)
			{
				// check if the hidden iframe into which the contents will be downloaded exists, if not, create it
				this.checkIFrame();

				// Update contents of the element
				// check wether the URL has changed or not, since the contents don't have to be reloaded to update the wrapping of text
				if (document.frames(this.getId()+"Target") &&
				   document.frames(this.getId()+"Target").location.href != this.mbr_source)
				{
					this.mbr_beforeHeightUpdate = -1;

					// Set the src of the IFrame
					document.frames(this.getId()+"Target").location.href = this.mbr_source;

					setTimeout("relativeLayerIPollIE('"+this.getId()+"')", 200);
				}
			}
		}
	}
};

RelativeLayer.prototype.finishContentsUpdate = function()
{
	if (op)
	{
		// NOT IMPLEMENTED
	}
	else if (mz)
	{
		if (!this.mbr_updateCompleted)
		{
			this.mbr_updateCompleted = true;
			this.mbr_node.innerHTML = document.getElementById(this.getId()+"Target").contentDocument.firstChild.innerHTML;
			
			this.updateAutoVisibleWidth();
			this.updateAutoVisibleHeight();
			this.updateAutoAvailableWidth();
			this.updateAutoAvailableHeight();
		}
	}
	else if (kq)
	{
		if (!this.mbr_updateCompleted)
		{
			this.mbr_updateCompleted = true;
			var var_innerhtml = document.getElementById(this.getId()+"Target").contentDocument.firstChild.innerHTML;
			// strip off the initial body tag since this makes Konqueror react weirdly
			// the contents are interpreted by konqueror and converted to a valid xhtml doc
			// the exact contents are thus not transferred
			if (var_innerhtml.indexOf("<BODY>") == 0)
			{
				var_innerhtml = var_innerhtml.substring("<BODY>".length);
			}
			this.mbr_node.innerHTML = var_innerhtml;
			
			this.updateAutoVisibleWidth();
			this.updateAutoVisibleHeight();
			this.updateAutoAvailableWidth();
			this.updateAutoAvailableHeight();
		}
	}
	else if (ns)
	{
		this.updateAutoVisibleWidth();
		this.updateAutoVisibleHeight();
		this.updateAutoAvailableWidth();
		this.updateAutoAvailableHeight();
	
		// redefine the drag events since Netscape loses them on document write
		if (this.setDragEvents)
		{
			this.setDragEvents();
		}
	}
	else if (ie)
	{
		// update the layer dimensions if they have to be calculate dynamically
		if (this.mbr_beforeHeightUpdate == -1)
		{
			this.mbr_beforeHeightUpdate = this.mbr_node.scrollHeight;
			this.css.width = this.calculateAvailableWidth()+"px";
		}
		else
		{
			this.updateAutoVisibleWidth();
			this.updateAutoVisibleHeight();
			this.updateAutoAvailableWidth();
			this.updateAutoAvailableHeight();
		}
	}
	
	glb_updating_layer = null;
	this.mbr_updateCompleted = true;
	
	// notify event listerens
	this.fireEvent("load");
};

RelativeLayer.prototype.replaceIFrame = function(par_source)
{
	if (kq)
	{
		var var_existing_iframe_div = document.getElementById(this.getId()+"TargetDiv");
		var var_iframe;
		var var_iframe_div;
		var_iframe = document.createElement("IFRAME");
		var_iframe.id = this.getId()+"Target";
		var_iframe.src = par_source;
		var_iframe.width = "300px";
		var_iframe.height = "100px";
		var_iframe_div = document.createElement("DIV");
		var_iframe_div.id = this.getId()+"TargetDiv";
		var_iframe_div.style.visibility = "hidden";
		var_iframe_div.style.position = "absolute";
		var_iframe_div.style.top = "-1000px";
		var_iframe_div.style.left = "-1000px";
		var_iframe_div.appendChild(var_iframe);
		
		if (!var_existing_iframe_div)
		{
			document.body.appendChild(var_iframe_div);
		}
		else
		{
			document.body.replaceChild(var_existing_iframe_div, var_iframe_div);
		}
	}
};

RelativeLayer.prototype.checkIFrame = function()
{
	if (mz)
	{
		var var_iframe = document.getElementById(this.getId()+"Target");
		if (!var_iframe)
		{
			var_iframe = document.createElement("IFRAME");
			var_iframe.id = this.getId()+"Target";
			var_iframe.src = "";
			var_iframe.width = "300px";
			var_iframe.height = "100px";
			var var_iframe_div = document.createElement("DIV");
			var_iframe_div.style.visibility = "hidden";
			var_iframe_div.style.position = "absolute";
			var_iframe_div.style.top = "-1000px";
			var_iframe_div.style.left = "-1000px";
			var_iframe_div.appendChild(var_iframe);
			document.body.appendChild(var_iframe_div);
		}
		else
		{
			var_iframe.src = "";
			var_iframe.innerHTML = "";
		}
	}
	else if (ie)
	{
		// Check if the IFrame exists already. It's done with a regexp since IE Mac apparently does allow you to check the existance of a non existing IFrame .. useful feature !
		var var_regexp = new RegExp(this.getId()+"Target","g");
		var var_iframe = String(document.body.innerHTML).match(var_regexp);

		// Check if it exists. If not create one in the document.
		// Create 1 IFrame per positioned DIV.
		if (var_iframe == null)
		{
			// Insert at the end of the document
			if (glb_platform == "mac")
			{
				document.body.insertAdjacentHTML("beforeEnd", '<IFRAME STYLE="visibility: hidden; display: none;" NAME="'+this.getId()+'Target" SRC=""></IFRAME>');
			}
			else
			{
				document.body.insertAdjacentHTML("beforeEnd", '<IFRAME STYLE="visibility: hidden;" NAME="'+this.getId()+'Target" SRC=""></IFRAME>');
			}
		}
		else
		{
			document.frames(this.getId()+"Target").location.href = "";
			document.frames(this.getId()+"Target").document.body.innerHTML = "<P>&nbsp;</P>";
		}
	}
};

function relativeLayerIPollIE(par_id)
{
	// check if the document was correctly downloaded and inserted in the frame (an empty frame contains '<P>&nbsp;</P>' strangely enough
	if (document.frames(par_id+"Target").document.readyState == 'complete' &&
	   document.frames(par_id+"Target").document.all.length > 1)
	{
		// IE 4.x mac can be very slow at creating all the DOM objects, if there's not enough memory, it even refuses to create them without giving a notice, this will prevent Javascript errors
		if (String(document.frames(par_id+"Target").document.body))
		{
			if (!String(document.frames(par_id+"Target").document.body.innerHTML).match(/\<P\>\&nbsp\;\<\/P\>/))
			{
				// Finished, copy the contents
				document.all(par_id).innerHTML = document.frames(par_id+"Target").document.body.innerHTML;
				document.all(par_id).style.mbr_relativeLayerObject.finishContentsUpdate();
				
				return;
			}
		}
	}
	
	setTimeout("relativeLayerIPollIE('"+par_id+"')", 200);
};
